@@ -51,35 +51,27 @@ from shenfun.spectralbase import SpectralBase, Transform, FuncWrap, \
 from shenfun.matrixbase import SparseMatrix
 from shenfun.optimization import optimizer
 from shenfun.config import config
-from shenfun.jacobi.recursions import n
+from shenfun.jacobi.recursions import n, half, cn
 
-__all__ = ['Orthogonal',
-           'ShenDirichlet',
-           'Phi1',
-           'Phi2',
-           'Phi3',
-           'Phi4',
-           'Heinrichs',
-           'ShenNeumann',
-           'CombinedShenNeumann',
-           'MikNeumann',
-           'ShenBiharmonic',
-           'UpperDirichlet',
-           'LowerDirichlet',
-           'UpperDirichletNeumann',
-           'LowerDirichletNeumann',
-           'ShenBiPolar',
-           'DirichletNeumann',
-           'NeumannDirichlet',
-           'Generic',
-           'BCGeneric',
-           'BCDirichlet',
-           'BCBiharmonic',
-           'BCNeumann',
-           'BCUpperDirichlet',
-           'BCLowerDirichlet',
-           'CompositeBase']
+bases = ['Orthogonal',
+         'ShenDirichlet',
+         'Heinrichs',
+         'ShenNeumann',
+         'CombinedShenNeumann',
+         'MikNeumann',
+         'ShenBiharmonic',
+         'UpperDirichlet',
+         'LowerDirichlet',
+         'UpperDirichletNeumann',
+         'LowerDirichletNeumann',
+         'ShenBiPolar',
+         'DirichletNeumann',
+         'NeumannDirichlet',
+         'Generic']
+bcbases = ['BCGeneric']
+testbases = ['Phi1', 'Phi2', 'Phi3', 'Phi4']
 
+__all__ = bases + bcbases + testbases
 
 #pylint: disable=abstract-method, not-callable, method-hidden, no-self-use, cyclic-import
 
@@ -152,12 +144,15 @@ class Orthogonal(SpectralBase):
         SpectralBase.__init__(self, N, quad=quad, domain=domain, dtype=dtype,
                               padding_factor=padding_factor, dealias_direct=dealias_direct,
                               coordinates=coordinates)
-        assert quad in ('GC', 'GL')
+        assert quad in ('GC', 'GL', 'GU')
+        self.alpha = -half
+        self.beta = -half
+        self.gn = cn
         if quad == 'GC':
             self._xfftn_fwd = functools.partial(fftw.dctn, type=2)
             self._xfftn_bck = functools.partial(fftw.dctn, type=3)
             self._xfftn_fwd.opts = self._xfftn_bck.opts = config['fftw']['dct']
-        elif quad == 'GL':
+        elif quad in ('GL', 'GU'):
             self._xfftn_fwd = functools.partial(fftw.dctn, type=1)
             self._xfftn_bck = functools.partial(fftw.dctn, type=1)
             self._xfftn_fwd.opts = self._xfftn_bck.opts = config['fftw']['dct']
@@ -324,6 +319,10 @@ class Orthogonal(SpectralBase):
     def short_name():
         return 'T'
 
+    def stencil_matrix(self, N=None):
+        N = self.N if N is None else N
+        return SparseMatrix({0: 1}, (N, N))
+
     def to_ortho(self, input_array, output_array=None):
         assert input_array.__class__.__name__ == 'Orthogonal'
         if output_array:
@@ -331,8 +330,15 @@ class Orthogonal(SpectralBase):
             return output_array
         return input_array
 
-    def get_orthogonal(self):
-        return self
+    def get_orthogonal(self, **kwargs):
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype,
+                 padding_factor=self.padding_factor,
+                 dealias_direct=self.dealias_direct,
+                 coordinates=self.coors.coordinates)
+        d.update(kwargs)
+        return Orthogonal(self.N, **d)
 
     def get_bc_basis(self):
         if self._bc_basis:
@@ -1362,6 +1368,7 @@ class LowerDirichlet(CompositeBase):
     def slice(self):
         return slice(0, self.N-1)
 
+
 class ShenBiPolar(CompositeBase):
     """Function space for the Biharmonic equation in polar coordinates
 
@@ -1847,6 +1854,10 @@ class BCBase(CompositeBase):
     def boundary_condition():
         return 'Apply'
 
+    @property
+    def is_boundary_basis(self):
+        return True
+
     def shape(self, forward_output=True):
         if forward_output:
             return self.stencil_matrix().shape[0]
@@ -1854,24 +1865,25 @@ class BCBase(CompositeBase):
             return self.N
 
     @property
-    def num_T(self):
+    def dim_ortho(self):
+        """Dimension of orthogonal space"""
         return self.stencil_matrix().shape[1]
 
     def slice(self):
         return slice(self.N-self.shape(), self.N)
 
     def vandermonde(self, x):
-        return n_cheb.chebvander(x, self.num_T-1)
+        return n_cheb.chebvander(x, self.dim_ortho-1)
 
     def _composite(self, V, argument=1):
         N = self.shape()
         P = np.zeros(V[:, :N].shape)
-        P[:] = np.tensordot(V[:, :self.num_T], self.stencil_matrix(), (1, 1))
+        P[:] = np.tensordot(V[:, :self.dim_ortho], self.stencil_matrix(), (1, 1))
         return P
 
     def sympy_basis(self, i=0, x=xp):
         M = self.stencil_matrix()
-        return np.sum(M[i]*np.array([sp.chebyshevt(j, x) for j in range(self.num_T)]))
+        return np.sum(M[i]*np.array([sp.chebyshevt(j, x) for j in range(self.dim_ortho)]))
 
     def evaluate_basis(self, x, i=0, output_array=None):
         x = np.atleast_1d(x)
@@ -1893,7 +1905,7 @@ class BCBase(CompositeBase):
         else:
             output_array.fill(0)
         M = self.stencil_matrix().T
-        output_array[:self.num_T] = np.dot(M, input_array[self.sl[self.slice()]])
+        output_array[:self.dim_ortho] = np.dot(M, input_array[self.sl[self.slice()]])
         return output_array
 
     def eval(self, x, u, output_array=None):
@@ -1901,55 +1913,13 @@ class BCBase(CompositeBase):
         output_array = v.eval(x, output_array=output_array)
         return output_array
 
-class BCDirichlet(BCBase):
+    def get_orthogonal(self, **kwargs):
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype)
+        d.update(kwargs)
+        return Orthogonal(self.dim_ortho, **d)
 
-    @staticmethod
-    def short_name():
-        return 'BCD'
-
-    def stencil_matrix(self, N=None):
-        return sp.Rational(1, 2)*np.array([[1, -1],
-                                           [1, 1]])
-
-class BCNeumann(BCBase):
-
-    @staticmethod
-    def short_name():
-        return 'BCN'
-
-    def stencil_matrix(self, N=None):
-        return sp.Rational(1, 8)*np.array([[0, 4, -1],
-                                           [0, 4, 1]])
-
-class BCBiharmonic(BCBase):
-
-    @staticmethod
-    def short_name():
-        return 'BCB'
-
-    def stencil_matrix(self, N=None):
-        return sp.Rational(1, 16)*np.array([[8, -9, 0, 1],
-                                            [2, -1, -2, 1],
-                                            [8, 9, 0, -1],
-                                            [-2, -1, 2, 1]])
-
-class BCUpperDirichlet(BCBase):
-
-    @staticmethod
-    def short_name():
-        return 'BCUD'
-
-    def stencil_matrix(self, N=None):
-        return sp.Rational(1, 2)*np.array([[1, 1]])
-
-class BCLowerDirichlet(BCBase):
-
-    @staticmethod
-    def short_name():
-        return 'BCLD'
-
-    def stencil_matrix(self, N=None):
-        return sp.Rational(1, 2)*np.array([[1, -1]])
 
 class BCGeneric(BCBase):
 
