@@ -18,13 +18,14 @@ different bases according to
     - T  = Orthogonal
     - CD = CompactDirichlet
     - CN = CompactNeumann
-    - BD = BCDirichlet
-    - BB = BCBiharmonic
+    - BCG = BCGeneric
     - P1 = Phi1
     - P2 = Phi2
+    - P3 = Phi3
+    - P4 = Phi4
 
-So a mass matrix using CompactDirichlet trial and Phi1 test is named
-BP1CDmat.
+So a mass matrix using CompactDirichlet trial and test is named
+BCDCDmat.
 
 All matrices in this module may be looked up using the 'mat' dictionary,
 which takes test and trialfunctions along with the number of derivatives
@@ -65,10 +66,9 @@ To see that this is in fact the BUUmat:
 
 from __future__ import division
 
-import functools
 import numpy as np
 import sympy as sp
-from shenfun.matrixbase import SpectralMatrix, SparseMatrix
+from shenfun.matrixbase import SpectralMatrix, SparseMatrix, SpectralMatDict
 from shenfun.la import TwoDMA
 from shenfun.chebyshev import bases as chebbases
 from . import bases
@@ -80,14 +80,39 @@ xp = sp.symbols('x', real=True, positive=True)
 U = bases.Orthogonal
 CD = bases.CompactDirichlet
 CN = bases.CompactNeumann
+UD = bases.UpperDirichlet
+LD = bases.LowerDirichlet
+CB = bases.CompositeBase
 P1 = bases.Phi1
 P2 = bases.Phi2
-BCD = bases.BCDirichlet
-BCB = bases.BCBiharmonic
+P3 = bases.Phi3
+P4 = bases.Phi4
+BCG = bases.BCGeneric
 
 SD = chebbases.ShenDirichlet
 SN = chebbases.ShenNeumann
 
+def get_UU(M, N, quad):
+    """Return main diagonal of :math:`(U_i, U_j)_w`
+
+    Parameters
+    ----------
+    M : int
+        The number of quadrature points in the test function
+    N : int
+        The number of quadrature points in the trial function
+    quad : str
+        Type of quadrature
+
+        - GC - Chebyshev-Gauss
+        - GU - Chebyshevu-Gauss
+    """
+    ll = np.pi/2
+    if quad == 'GC' and N >= M:
+        ll = np.ones(min(M, N), dtype=float)*ll
+        ll[-1] = 2*ll[-1]
+    return ll
+
 class BUUmat(SpectralMatrix):
     r"""Mass matrix :math:`B=(b_{kj}) \in \mathbb{R}^{M \times N}`, where
 
@@ -99,10 +124,12 @@ class BUUmat(SpectralMatrix):
     dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test, trial = self.testfunction, self.trialfunction
         assert isinstance(test[0], U)
         assert isinstance(trial[0], U)
-        SpectralMatrix.__init__(self, {0: np.pi/2}, test, trial, scale=scale, measure=measure)
+        d = get_UU(test[0].N, trial[0].N, test[0].quad)
+        return {0: d}
 
 class BP1SDmat(SpectralMatrix):
     r"""Mass matrix :math:`B=(b_{kj}) \in \mathbb{R}^{M \times N}`, where
@@ -116,21 +143,23 @@ class BP1SDmat(SpectralMatrix):
     trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test, trial = self.testfunction, self.trialfunction
         from shenfun.jacobi.recursions import Lmat, half, cn
         assert isinstance(test[0], P1)
         assert isinstance(trial[0], SD)
+        assert test[0].quad == 'GU'
         N = test[0].N-2
         K = trial[0].stencil_matrix()
         K.shape = (N, N+2)
-        K = K.diags('csr')
         B2 = Lmat(2, 0, 2, N, N+2, -half, -half, cn) # B^{(2)_{(2)}}
         if not test[0].is_scaled():
             k = np.arange(N+2)
             B2 = SparseMatrix({0: (k[:-2]+2)}, (N, N)).diags('csr')*B2
-        M = B2 * K.T
+        M = B2 * K.diags('csr').T
+        K.shape = (N+2, N+2)
         d = {-2: M.diagonal(-2), 0: M.diagonal(0), 2: M.diagonal(2), 4: M.diagonal(4)}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
 class AP1SDmat(SpectralMatrix):
     r"""Stiffness matrix :math:`A=(a_{kj}) \in \mathbb{R}^{M \times N}`, where
@@ -144,14 +173,15 @@ class AP1SDmat(SpectralMatrix):
     trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test, trial = self.testfunction, self.trialfunction
         assert isinstance(test[0], P1)
         assert isinstance(trial[0], SD)
         d = {0: -1, 2: 1}
         if not test[0].is_scaled():
             k = np.arange(test[0].N-2)
             d = {0: -(k+2), 2: k[:-2]+2}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
     def get_solver(self):
         return TwoDMA
@@ -168,21 +198,22 @@ class BP1SNmat(SpectralMatrix):
     trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
         from shenfun.jacobi.recursions import Lmat, half, cn
+        test, trial = self.testfunction, self.trialfunction
         assert isinstance(test[0], P1)
         assert isinstance(trial[0], SN)
         N = test[0].N-2
         K = trial[0].stencil_matrix()
         K.shape = (N, N+2)
-        K = K.diags('csr')
         B2 = Lmat(2, 0, 2, N, N+2, -half, -half, cn) # B^{(2)_{(2)}}
         if not test[0].is_scaled():
             k = np.arange(test[0].N)
             B2 = SparseMatrix({0: (k[:-2]+2)}, (N, N)).diags('csr')*B2
-        M = B2 * K.T
+        M = B2 * K.diags('csr').T
+        K.shape = (N+2, N+2)
         d = {-2: M.diagonal(-2), 0: M.diagonal(0), 2: M.diagonal(2), 4: M.diagonal(4)}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
 class AP1SNmat(SpectralMatrix):
     r"""Stiffness matrix :math:`A=(a_{kj}) \in \mathbb{R}^{M \times N}`, where
@@ -196,50 +227,25 @@ class AP1SNmat(SpectralMatrix):
     trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test, trial = self.testfunction, self.trialfunction
         assert isinstance(test[0], P1)
         assert isinstance(trial[0], SN)
         k = np.arange(test[0].N-2)
         d = {0: -(k/(k+2))**2, 2: 1}
         if not test[0].is_scaled():
             d = {0: -k**2/(k+2), 2: k[:-2]+2}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
     def get_solver(self):
         return TwoDMA
 
-class _Chebumatrix(SpectralMatrix):
-    def __init__(self, test, trial, scale=1, measure=1):
-        SpectralMatrix.__init__(self, {}, test, trial, scale=scale, measure=measure)
-
-class _ChebuMatDict(dict):
-    """Dictionary of inner product matrices
-
-    Matrices that are missing keys are generated from Vandermonde type
-    computations.
-
-    """
-
-    def __missing__(self, key):
-        measure = 1 if len(key) == 2 else key[3]
-        c = functools.partial(_Chebumatrix, measure=measure)
-        self[key] = c
-        return c
-
-    def __getitem__(self, key):
-        matrix = dict.__getitem__(self, key)
-        #assert key[0][1] == 0, 'Test cannot be differentiated (weighted space)'
-        return matrix
-
-
-
 # Define dictionary to hold all predefined matrices
 # When looked up, missing matrices will be generated automatically
-mat = _ChebuMatDict({
+mat = SpectralMatDict({
     ((U,  0), (U,  0)): BUUmat,
     ((P1, 0), (SD, 0)): BP1SDmat,
     ((P1, 0), (SN, 0)): BP1SNmat,
     ((P1, 0), (SD, 2)): AP1SDmat,
     ((P1, 0), (SN, 2)): AP1SNmat,
-
 })
