@@ -2,14 +2,11 @@
 Module for handling Fourier diagonal matrices
 """
 from __future__ import division
-
-#__all__ = ['mat']
-
 import functools
 from numbers import Number
 import numpy as np
 import sympy as sp
-from shenfun.matrixbase import SpectralMatrix
+from shenfun.matrixbase import SpectralMatrix, SpectralMatDict
 from shenfun import la
 from . import bases
 
@@ -29,7 +26,8 @@ class Acos2mat(SpectralMatrix):
     where test and trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test = self.testfunction
         k = test[0].wavenumbers(bcast=False, scaled=False, eliminate_highest_freq=False)
         N = test[0].N
         d = {0: -0.5*k**2,
@@ -37,7 +35,7 @@ class Acos2mat(SpectralMatrix):
              -2: -0.25*k[:-2]**2,
              N-2: -0.25*k[-2:]**2,
              -(N-2): -0.25*k[:2]**2}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
 
 class Acosmat(SpectralMatrix):
@@ -50,13 +48,14 @@ class Acosmat(SpectralMatrix):
     where test and trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test = self.testfunction
         k = test[0].wavenumbers(bcast=False, scaled=False, eliminate_highest_freq=False)
         N = test[0].N
         d = {1: -0.5*k[1:]**2,
              -1: -0.5*k[:-1]**2,
              N-1: -0.5*k[-1]**2}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
 
 class Csinmat(SpectralMatrix):
@@ -69,13 +68,14 @@ class Csinmat(SpectralMatrix):
     where test and trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test = self.testfunction
         k = test[0].wavenumbers(bcast=False, scaled=False, eliminate_highest_freq=False)
         N = test[0].N
         d = {1: -0.5*k[1:],
              -1: 0.5*k[:-1],
              N-1: -0.5}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
 
 class Csincosmat(SpectralMatrix):
@@ -88,15 +88,15 @@ class Csincosmat(SpectralMatrix):
     where test and trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test = self.testfunction
         k = test[0].wavenumbers(bcast=False, scaled=False, eliminate_highest_freq=False)
         N = test[0].N
         d = {2: -0.25*k[2:],
              -2: 0.25*k[:-2],
              N-2: 0.25*k[-2:],
              -(N-2): -0.25*k[:2]}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
-
+        return d
 
 class Bcos2mat(SpectralMatrix):
     r"""Matrix for :math:`B=(b_{kj}) \in \mathbb{R}^{M \times N}`, where
@@ -108,14 +108,15 @@ class Bcos2mat(SpectralMatrix):
     where test and trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test = self.testfunction
         N = test[0].N
         d = {0: 0.5,
              2: 0.25,
              -2: 0.25,
              N-2: 0.25,
              -(N-2): 0.25}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
 
 class Bcosmat(SpectralMatrix):
@@ -128,20 +129,31 @@ class Bcosmat(SpectralMatrix):
     where test and trial spaces have dimensions of M and N, respectively.
 
     """
-    def __init__(self, test, trial, scale=1, measure=1):
+    def assemble(self):
+        test = self.testfunction
         N = test[0].N
         d = {1: 0.5,
              -1: 0.5,
              N-1: 0.5,
              -(N-1): 0.5}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        return d
 
+class FourierMatDict(SpectralMatDict):
+    def __missing__(self, key):
+        measure = 1 if len(key) == 2 else key[2]
+        c = functools.partial(FourierMatrix, measure=measure)
+        self[key] = c
+        return c
 
-class _Fouriermatrix(SpectralMatrix):
-    def __init__(self, test, trial, scale=1, measure=1):
+class FourierMatrix(SpectralMatrix):
+    def __init__(self, test, trial, scale=1, measure=1, assemble=None):
+        SpectralMatrix.__init__(self, test, trial, scale=scale, measure=measure, assemble=assemble)
+
+    def assemble(self):
+        test, trial = self.testfunction, self.trialfunction
         N = test[0].N
         d = {}
-        if isinstance(measure, Number):
+        if isinstance(self.measure, Number):
             k = test[0].wavenumbers(N, scaled=False)
             if isinstance(test[1], (int, np.integer)):
                 k_test, k_trial = test[1], trial[1]
@@ -162,7 +174,9 @@ class _Fouriermatrix(SpectralMatrix):
                 d = {0: val*test[0].domain_factor()}
             else:
                 d = {0: test[0].domain_factor()}
-        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+        else:
+            d = None
+        return d
 
     def solve(self, b, u=None, axis=0, constraints=()):
         if self.measure == 1:
@@ -192,33 +206,14 @@ class _Fouriermatrix(SpectralMatrix):
         return la.Solver(self)(b, u=u, axis=axis, constraints=constraints)
 
 
-class _FourierMatDict(dict):
-    """Dictionary of inner product matrices.
-
-    Matrices that are missing keys are generated. All Fourier matrices are
-    diagonal.
-
-    """
-
-    def __missing__(self, key):
-        measure = 1 if len(key) == 2 else key[3]
-        c = functools.partial(_Fouriermatrix, measure=measure)
-        self[key] = c
-        return c
-
-    def __getitem__(self, key):
-        matrix = dict.__getitem__(self, key)
-        return matrix
-
-
-mat = _FourierMatDict({
-    ((C2C, 0), (C2C, 2), (0, 2*np.pi), sp.cos(xp)**2): functools.partial(Acos2mat, measure=sp.cos(xp)**2),
-    ((C2C, 0), (C2C, 2), (0, 2*np.pi), sp.cos(xp)): functools.partial(Acosmat, measure=sp.cos(xp)),
-    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(xp)): functools.partial(Csinmat, measure=sp.sin(xp)),
-    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(2*xp)/2): functools.partial(Csincosmat, measure=sp.sin(2*xp)/2),
-    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(xp)*sp.cos(xp)): functools.partial(Csincosmat, measure=sp.sin(xp)*sp.cos(xp)),
-    ((C2C, 0), (C2C, 0), (0, 2*np.pi), sp.cos(xp)**2): functools.partial(Bcos2mat, measure=sp.cos(xp)**2),
-    ((C2C, 0), (C2C, 0), (0, 2*np.pi), sp.cos(xp)): functools.partial(Bcosmat, measure=sp.cos(xp)),
+mat = FourierMatDict({
+    ((C2C, 0), (C2C, 2), sp.cos(xp)**2): Acos2mat,
+    ((C2C, 0), (C2C, 2), sp.cos(xp)): Acosmat,
+    ((C2C, 0), (C2C, 1), sp.sin(xp)): Csinmat,
+    ((C2C, 0), (C2C, 1), sp.sin(2*xp)/2): Csincosmat,
+    ((C2C, 0), (C2C, 1), sp.sin(xp)*sp.cos(xp)): Csincosmat,
+    ((C2C, 0), (C2C, 0), sp.cos(xp)**2): Bcos2mat,
+    ((C2C, 0), (C2C, 0), sp.cos(xp)): Bcosmat,
 })
 
-#mat = _FourierMatDict({})
+#mat = FourierMatDict({})
