@@ -10,14 +10,17 @@ from shenfun.spectralbase import SpectralBase, Transform, islicedict, \
 
 #pylint: disable=method-hidden,no-else-return,not-callable,abstract-method,no-member,cyclic-import
 
-__all__ = ['Orthogonal',
-           'CompactDirichlet',
-           'CompactNeumann',
-           'Generic',
-           'CompositeBase']
+bases = ['Orthogonal',
+         'CompactDirichlet',
+         'CompactNeumann',
+         'Generic']
+bcbases = ['BCGeneric']
+testbases = []
+__all__ = bases + bcbases
 
 xp = sp.Symbol('x', real=True)
 
+
 class Orthogonal(SpectralBase):
     r"""Function space for a regular Laguerre series
 
@@ -121,6 +124,10 @@ class Orthogonal(SpectralBase):
             return (lambda i: -i-sp.S.Half, None)
         raise NotImplementedError
 
+    def stencil_matrix(self, N=None):
+        N = self.N if N is None else N
+        return SparseMatrix({0: 1}, (N, N))
+
     def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         if x is None:
             x = self.mesh(False, False)
@@ -188,8 +195,15 @@ class Orthogonal(SpectralBase):
     def is_orthogonal(self):
         return True
 
-    def get_orthogonal(self):
-        return self
+    def get_orthogonal(self, **kwargs):
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype,
+                 padding_factor=self.padding_factor,
+                 dealias_direct=self.dealias_direct,
+                 coordinates=self.coors.coordinates)
+        d.update(kwargs)
+        return Orthogonal(self.N, **d)
 
     @staticmethod
     def short_name():
@@ -464,6 +478,10 @@ class BCBase(CompositeBase):
     def boundary_condition():
         return 'Apply'
 
+    @property
+    def is_boundary_basis(self):
+        return True
+
     def shape(self, forward_output=True):
         if forward_output:
             return self.stencil_matrix().shape[0]
@@ -471,26 +489,26 @@ class BCBase(CompositeBase):
             return self.N
 
     @property
-    def num_T(self):
+    def dim_ortho(self):
         return self.stencil_matrix().shape[1]
 
     def slice(self):
         return slice(self.N-self.shape(), self.N)
 
     def vandermonde(self, x):
-        V = lag.lagvander(x, self.num_T-1)
+        V = lag.lagvander(x, self.dim_ortho-1)
         V *= np.exp(-x/2)[:, None]
         return V
 
     def _composite(self, V, argument=1):
         N = self.shape()
         P = np.zeros(V[:, :N].shape)
-        P[:] = np.tensordot(V[:, :self.num_T], self.stencil_matrix(), (1, 1))
+        P[:] = np.tensordot(V[:, :self.dim_ortho], self.stencil_matrix(), (1, 1))
         return P
 
     def sympy_basis(self, i=0, x=xp):
         M = self.stencil_matrix()
-        return np.sum(M[i]*np.array([sp.laguerre(j, x)*sp.exp(-x/2) for j in range(self.num_T)]))
+        return np.sum(M[i]*np.array([sp.laguerre(j, x)*sp.exp(-x/2) for j in range(self.dim_ortho)]))
 
     def evaluate_basis(self, x, i=0, output_array=None):
         x = np.atleast_1d(x)
@@ -521,6 +539,13 @@ class BCBase(CompositeBase):
         output_array = v.eval(x, output_array=output_array)
         return output_array
 
+    def get_orthogonal(self, **kwargs):
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype)
+        d.update(kwargs)
+        return Orthogonal(self.dim_ortho, **d)
+
 class BCGeneric(BCBase):
 
     @staticmethod
