@@ -21,6 +21,7 @@ from numbers import Number
 import sympy as sp
 import numpy as np
 from mpi4py_fft import fftw
+from shenfun import config
 from .utilities import CachedArrayDict, split
 from .coordinates import Coordinates
 work = CachedArrayDict()
@@ -29,7 +30,6 @@ class SpectralBase:
     """Abstract base class for all spectral function spaces
     """
     # pylint: disable=method-hidden, too-many-instance-attributes
-
     def __init__(self, N, quad='', padding_factor=1, domain=(-1., 1.), dtype=None,
                  dealias_direct=False, coordinates=None):
         self.N = N
@@ -38,6 +38,9 @@ class SpectralBase:
         self.axis = 0
         self.bc = None
         self._bc_basis = None
+        self.alpha = None  # Jacobi parameter
+        self.beta = None   # Jacobi parameter
+        self.gn = None     # Jacobi scaling function
         self.padding_factor = padding_factor
         if padding_factor != 1:
             self.padding_factor = np.floor(N*padding_factor)/N if N > 0 else 1
@@ -677,6 +680,25 @@ class SpectralBase:
             x = a + (x-c)/self.domain_factor()
         return x
 
+    def map_expression_true_domain(self, f, x=None):
+        """Return expression `f` mapped to true domain as a function
+        of the reference coordinate
+
+        Parameters
+        ----------
+        f : Sympy expression
+        x : Sympy symbol, optional
+            coordinate
+        """
+        if not self.domain == self.reference_domain():
+            f = sp.sympify(f)
+            if len(f.free_symbols) == 1:
+                if x is None:
+                    x = f.free_symbols.pop()
+                xm = self.map_true_domain(x)
+                f = f.replace(x, xm)
+        return f
+
     def sympy_basis(self, i=0, x=sp.Symbol('x', real=True)):
         """Return basis function `i` as sympy function
 
@@ -812,6 +834,10 @@ class SpectralBase:
     def has_nonhomogeneous_bcs(self):
         return False
 
+    @property
+    def is_boundary_basis(self):
+        return False
+
     @property
     def rank(self):
         """Return rank of function space
@@ -873,7 +899,10 @@ class SpectralBase:
                 x0 = x0.pop()
                 x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
                 dx = dx.subs(x0, x)
-            key = key + (self.domain, dx)
+                if self.domain != self.reference_domain():
+                    xm = self.map_true_domain(x)
+                    dx = dx.replace(x, xm)
+            key = key + (dx,)
 
         return mat[key]
 
@@ -902,7 +931,7 @@ class SpectralBase:
         msi = dv[msx]
         return inner_product((self, 0), (self.get_bc_basis(), 0), msi)
 
-    def get_measured_weights(self, N=None, measure=1):
+    def get_measured_weights(self, N=None, measure=1, map_true_domain=False):
         """Return weights times ``measure``
 
         Parameters
@@ -913,7 +942,7 @@ class SpectralBase:
         """
         if N is None:
             N = self.shape(False)
-        xm, wj = self.mpmath_points_and_weights(N, map_true_domain=True)
+        xm, wj = self.mpmath_points_and_weights(N, map_true_domain=map_true_domain)
         if measure == 1:
             return wj
 
@@ -1136,7 +1165,7 @@ class SpectralBase:
         T = T.get_refined(T.N - trailing_zeros)
         return T
 
-    def get_orthogonal(self):
+    def get_orthogonal(self, **kwargs):
         """Return orthogonal space (otherwise as self)
 
         Returns
@@ -1218,9 +1247,11 @@ def getCompositeBase(Orthogonal):
             Orthogonal.__init__(self, *args, **kwargs)
             if bc is not None:
                 from shenfun.tensorproductspace import BoundaryValues
-                from shenfun.forms.arguments import BoundaryConditions
+                if isinstance(bc, dict) and not isinstance(bc, BoundaryConditions):
+                    bc = BoundaryConditions(bc, domain=kwargs['domain'])
                 if isinstance(bc, (tuple, list)):
                     bc = BoundaryConditions(bc, domain=kwargs['domain'])
+
                 assert isinstance(bc, BoundaryConditions)
                 self.bcs = bc
                 self.bc = BoundaryValues(self, bc=bc)
@@ -1259,17 +1290,18 @@ def getCompositeBase(Orthogonal):
         def is_orthogonal(self):
             return False
 
-        def get_orthogonal(self):
-            kwargs = dict(quad=self.quad,
-                          domain=self.domain,
-                          dtype=self.dtype,
-                          padding_factor=self.padding_factor,
-                          dealias_direct=self.dealias_direct,
-                          coordinates=self.coors.coordinates)
+        def get_orthogonal(self, **kwargs):
+            d = dict(quad=self.quad,
+                     domain=self.domain,
+                     dtype=self.dtype,
+                     padding_factor=self.padding_factor,
+                     dealias_direct=self.dealias_direct,
+                     coordinates=self.coors.coordinates)
             for kw in ('alpha', 'beta'):
                 if hasattr(self, kw):
-                    kwargs[kw] = getattr(self, kw)
-            return Orthogonal(self.N, **kwargs)
+                    d[kw] = getattr(self, kw)
+            d.update(kwargs)
+            return Orthogonal(self.N, **d)
 
         @property
         def has_nonhomogeneous_bcs(self):
@@ -1748,28 +1780,29 @@ class slicedict(dict):
         return dict.__getitem__(self, self.__keytransform__(key))
 
 
-def inner_product(test, trial, measure=1):
+def inner_product(test, trial, measure=1, assemble=None):
     """Return 1D weighted inner product of bilinear form
 
     Parameters
     ----------
-        test : 2-tuple of (Basis, integer)
-            Basis is any of the classes from
+    test : 2-tuple of (Basis, integer)
+        Basis is any of the classes from
 
-            - :mod:`.chebyshev.bases`
-            - :mod:`.chebyshevu.bases`
-            - :mod:`.legendre.bases`
-            - :mod:`.ultraspherical.bases`
-            - :mod:`.fourier.bases`
-            - :mod:`.laguerre.bases`
-            - :mod:`.hermite.bases`
-            - :mod:`.jacobi.bases`
+        - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
+        - :mod:`.legendre.bases`
+        - :mod:`.ultraspherical.bases`
+        - :mod:`.fourier.bases`
+        - :mod:`.laguerre.bases`
+        - :mod:`.hermite.bases`
+        - :mod:`.jacobi.bases`
 
-            The integer determines the number of times the basis is
-            differentiated. The test represents the matrix row
-        trial : 2-tuple of (Basis, integer)
-            Like test
-        measure: function of coordinate, optional
+        The integer determines the number of times the basis is
+        differentiated. The test represents the matrix row
+    trial : 2-tuple of (Basis, integer)
+        Like test
+    measure: function of coordinate, optional
+        The measure is in physical coordinates, not in the reference domain.
 
     Note
     ----
@@ -1790,31 +1823,38 @@ def inner_product(test, trial, measure=1):
     >>> [np.all(B[k] == v) for k, v in d.items()]
     [True, True, True]
     """
-    #assert trial[0].__module__ == test[0].__module__
     key = ((test[0].__class__, test[1]), (trial[0].__class__, trial[1]))
-    sc = 1
+    mat = test[0]._get_mat()
+
     if measure != 1:
+        # replace y, z with x if multidimensional
         x0 = measure.free_symbols
         assert len(x0) == 1
         x0 = x0.pop()
         x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
         measure = measure.subs(x0, x)
-        try:
-            if measure.subs(x, (test[0].domain[0]+test[0].domain[1])/2).evalf() < 0:
-                sc = -1
-                measure *= sc
-        except TypeError:
-            pass
+
+        measure = test[0].map_expression_true_domain(measure, x)
+
         if measure.is_polynomial():
             measure = sp.simplify(measure)
 
-        key = key + (test[0].domain, measure)
+        if key + (measure,) in mat:
+            A = mat[key+(measure,)](test, trial, assemble=assemble)
+
+        else:
+            # By mapping measure to true domain, the expression may be split
+            B = []
+            for dv in split(measure, expand=False):
+                sci = dv['coeff']
+                msi = dv['x']
+                newkey = key + (msi,)
+                B.append(mat[newkey](test, trial, assemble=assemble))
+                B[-1].scale *= sci
+            A = B[0] if len(B) == 1 else np.sum(np.array(B, dtype=object))
+    else:
+        A = mat[key](test, trial, assemble=assemble)
 
-    mat = test[0]._get_mat()
-    A = mat[key](test, trial)
-    A.scale *= sc
-    #if not test[0].domain_factor() == 1:
-    #    A.scale *= test[0].domain_factor()**(test[1]+trial[1]-1)
     return A
 
 class FuncWrap:
