@@ -3,7 +3,6 @@ Module for implementation of the :class:`.TensorProductSpace` class and
 related methods.
 """
 from numbers import Number
-import functools
 import sympy as sp
 import numpy as np
 from shenfun.fourier.bases import R2C, C2C
@@ -32,7 +31,7 @@ class ScalarTransform(Transform):
     pencil : list of two pencil objects
         The two pencils represent the input and final output configuration of
         the distributed global arrays
-    T : TensorProductSpace
+    T : :class:`.TensorProductSpace`
     """
     def __init__(self, xfftn, transfer, pencil, T=None):
         assert len(xfftn) == len(transfer) + 1 and len(pencil) == 2
@@ -49,12 +48,9 @@ class ScalarTransform(Transform):
 
     @property
     def local_mesh(self):
-        """Return local mesh"""
         return self._T.local_mesh(True)
 
     def get_measured_input_array(self):
-        """Weigh input array with integral measure
-        """
         dx = self.sg
         mesh = self.local_mesh
         if dx == 1:
@@ -82,14 +78,14 @@ class ScalarTransform(Transform):
         output_array : array, optional
         kw : dict
             parameters to serial transforms
-            Note in particular that the keyword 'normalize'=True/False can be
+            Note in particular that the keyword ``'normalize'=True/False`` can be
             used to turn normalization on or off. Default is to enable
             normalization for forward transforms and disable it for backward.
 
         Note
         ----
         If input_array/output_array are not given, then use predefined arrays
-        as planned with serial transform object _xfftn.
+        as planned with serial transform object ``_xfftn``.
 
         """
         if input_array is not None:
@@ -122,7 +118,7 @@ class ForwardTransform(ScalarTransform):
     pencil : list of two pencil objects
         The two pencils represent the input and final output configuration of
         the distributed global arrays
-    T : TensorProductSpace
+    T : :class:`.TensorProductSpace`
     """
     def __call__(self, input_array=None, output_array=None, **kw):
         if input_array is not None:
@@ -168,7 +164,7 @@ class TensorProductSpace(PFFT):
     ----------
     comm : MPI communicator
     bases : list
-        List of 1D bases
+        List of 1D bases (:class:`.SpectralBase`)
     axes : tuple of ints, optional
         A tuple containing the order of which to perform transforms.
         Last item is transformed first. Defaults to range(len(bases))
@@ -187,8 +183,8 @@ class TensorProductSpace(PFFT):
     coordinates : 2- or 3-tuple, optional
         Map for curvilinear coordinatesystem.
         First tuple are the coordinate variables in the new coordinate system
-        Second tuple are the Cartesian coordinates as functions of the variables
-        in the first tuple. Example::
+        Second tuple are the Cartesian coordinates as functions of the
+        variables in the first tuple. Example::
 
             psi = (theta, r) = sp.symbols('x,y', real=True, positive=True)
             rv = (r*sp.cos(theta), r*sp.sin(theta))
@@ -196,17 +192,17 @@ class TensorProductSpace(PFFT):
         where psi and rv are the first and second tuples, respectively.
         If a third item is provided with the tuple, then this third item
         is used as an additional assumption. For example, it is necessary
-        to provide the assumption `sympy.Q.positive(sympy.sin(theta))`, such
-        that sympy can evaluate `sqrt(sympy.sin(theta)**2)` to `sympy.sin(theta)`
-        and not `Abs(sympy.sin(theta))`. Different coordinates may require
-        different assumptions to help sympy when computing basis functions
-        etc.
+        to provide the assumption ``sympy.Q.positive(sympy.sin(theta))``,
+        such that sympy can evaluate ``sqrt(sympy.sin(theta)**2)`` to
+        ``sympy.sin(theta)`` and not ``Abs(sympy.sin(theta))``. Different
+        coordinates may require different assumptions to help sympy when
+        computing basis functions etc. See :class:`~shenfun.coordinates.Coordinates`.
     modify_spaces_inplace : bool, optional
         Whether or not a copy should be made of the input functionspaces.
         If True, then the input spaces will be modified inplace.
     kw : dict, optional
         Dictionary that can be used to plan transforms. Input to method
-        `plan` for the bases.
+        ``plan`` for the bases.
 
     """
     def __init__(self, comm, bases, axes=None, dtype=None, slab=False,
@@ -370,12 +366,12 @@ class TensorProductSpace(PFFT):
 
         Parameters
         ----------
-            pencil : Pencil
-                The distribution in spectral space
-            dtype : Numpy dtype
-                The type of data in spectral space
-            kw : dict
-                Any parameters for planning transforms
+        pencil : :class:`.Pencil`
+            The distribution in spectral space
+        dtype : Numpy dtype
+            The type of data in spectral space
+        kw : dict
+            Any parameters for planning transforms
 
         Note
         ----
@@ -439,6 +435,14 @@ class TensorProductSpace(PFFT):
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         """Return space (otherwise as self) to be used for dealiasing
+
+        Parameters
+        ----------
+        padding_factor : number, optional
+            Scale the number of quadrature points in self with this number
+        dealias_direct : bool, optional
+            Used only if ``padding_factor=1``. Sets the 1/3 highest frequencies
+            to zeros.
         """
         if isinstance(padding_factor, Number):
             padding_factor = (padding_factor,)*len(self)
@@ -510,7 +514,6 @@ class TensorProductSpace(PFFT):
         return TensorProductSpace(comm, bases)
 
     def dtype(self, forward_output=False):
-        """Return datatype function space is planned for"""
         if hasattr(self, 'forward'):
             return PFFT.dtype(self, forward_output)
         return self._dtype
@@ -735,7 +738,7 @@ class TensorProductSpace(PFFT):
         return output_array
 
     def wavenumbers(self, scaled=False, eliminate_highest_freq=False):
-        """Return list of wavenumbers of TensorProductSpace
+        """Return list of wavenumbers
 
         Parameters
         ----------
@@ -754,13 +757,13 @@ class TensorProductSpace(PFFT):
 
     def local_wavenumbers(self, broadcast=False, scaled=False,
                           eliminate_highest_freq=False):
-        """Return list of local wavenumbers of TensorProductSpace
+        """Return list of local wavenumbers
 
         Parameters
         ----------
         broadcast : bool, optional
             Broadcast returned wavenumber arrays to actual
-            dimensions of TensorProductSpace
+            dimensions of :class:`.TensorProductSpace`
         scaled : bool, optional
             Scale wavenumbers with size of box
         eliminate_highest_freq : bool, optional
@@ -780,7 +783,7 @@ class TensorProductSpace(PFFT):
 
     def mesh(self, uniform=False):
         """Return list of 1D physical meshes for each dimension of
-        TensorProductSpace
+        :class:`.TensorProductSpace`
 
         Parameters
         ----------
@@ -794,7 +797,7 @@ class TensorProductSpace(PFFT):
 
     def local_mesh(self, broadcast=False, uniform=False):
         """Return list of local 1D physical meshes for each dimension of
-        TensorProductSpace
+        :class:`.TensorProductSpace`
 
         Parameters
         ----------
@@ -815,7 +818,7 @@ class TensorProductSpace(PFFT):
         return lm
 
     def local_cartesian_mesh(self, uniform=False):
-        """Return Cartesian mesh
+        """Return local Cartesian mesh
 
         Parameters
         ----------
@@ -862,8 +865,8 @@ class TensorProductSpace(PFFT):
 
     @property
     def use_fixed_gauge(self):
-        """Return True if TensorProductSpace contains only spaces with Neumann
-        boundary conditions.
+        """Return True if :class:`.TensorProductSpace` contains only spaces
+        with Neumann boundary conditions.
         """
         for base in self.bases:
             if base.boundary_condition() != 'Neumann':
@@ -873,7 +876,7 @@ class TensorProductSpace(PFFT):
         return True
 
     def global_shape(self, forward_output=False):
-        """Return global shape of arrays for TensorProductSpace
+        """Return global shape of arrays for :class:`.TensorProductSpace`
 
         Parameters
         ----------
@@ -886,12 +889,12 @@ class TensorProductSpace(PFFT):
         return tuple([base.shape(forward_output) for base in self])
 
     def mask_nyquist(self, u_hat, mask=None):
-        """Return Function `u_hat` with zero Nyquist coefficients
+        """Return :class:`.Function` ``u_hat`` with zero Nyquist coefficients
 
         Parameters
         ----------
         u_hat : array
-            Function to be masked
+            :class:`.Function` to be masked
         mask : array or None, optional
             mask array, if not provided then get the mask by calling
             :func:`get_mask_nyquist`
@@ -931,7 +934,7 @@ class TensorProductSpace(PFFT):
         return mask
 
     def get_measured_array(self, u):
-        """Weigh Array `u` with integral measure
+        """Weigh Array ``u`` with integral measure
 
         Parameters
         ----------
@@ -960,7 +963,7 @@ class TensorProductSpace(PFFT):
 
     @property
     def rank(self):
-        """Return tensor rank of TensorProductSpace"""
+        """Return tensor rank of :class:`.TensorProductSpace`"""
         return 0
 
     @property
@@ -968,20 +971,19 @@ class TensorProductSpace(PFFT):
         return 0
 
     def __len__(self):
-        """Return dimension of TensorProductSpace"""
+        """Return dimension of :class:`.TensorProductSpace`"""
         return len(self.bases)
 
     def num_components(self):
-        """Return number of scalar spaces in TensorProductSpace"""
+        """Return number of scalar spaces in :class:`.TensorProductSpace`"""
         return 1
 
     @property
     def dimensions(self):
-        """Return dimension of TensorProductSpace"""
+        """Return dimension of :class:`.TensorProductSpace`"""
         return self.__len__()
 
     def get_nonperiodic_axes(self):
-        """Return list of axes that are not periodic"""
         axes = []
         for axis, base in enumerate(self):
             if not base.family() == 'fourier':
@@ -989,7 +991,7 @@ class TensorProductSpace(PFFT):
         return np.array(axes)
 
     def get_nondiagonal_axes(self):
-        """Return list of axes that **may** contain non-diagonal matrices"""
+        # Return list of axes that **may** contain non-diagonal matrices
         axes = []
         if isinstance(self.coors.hi.prod(), sp.Expr):
             x = 'xyzrs'
@@ -1007,7 +1009,6 @@ class TensorProductSpace(PFFT):
         return np.array(axes)
 
     def get_diagonal_axes(self):
-        """Return list of axes that contain diagonal matrices"""
         return np.setxor1d(self.get_nondiagonal_axes(), range(self.dimensions)).astype(int)
 
     def get_nonhomogeneous_axes(self):
@@ -1045,7 +1046,7 @@ class TensorProductSpace(PFFT):
 
         Returns
         -------
-        TensorProductSpace
+        :class:`.TensorProductSpace`
             A new space with adaptively found number of quadrature points
 
         Note
@@ -1115,7 +1116,7 @@ class TensorProductSpace(PFFT):
         Parameters
         ----------
 
-        space : TensorProductSpace
+        space : :class:`.TensorProductSpace`
             The space compared to
 
         Note
@@ -1144,7 +1145,7 @@ class TensorProductSpace(PFFT):
             return self.forward.input_pencil.subshape
         return self.forward.output_array.shape
 
-    def get_ndiag_cum_dofs(self):
+    def _get_ndiag_cum_dofs(self):
         """Return the cumulative sum of degrees of freedom along nondiagonal axes"""
         dims = np.array(self.dims())
         daxes = self.get_diagonal_axes()
@@ -1152,7 +1153,7 @@ class TensorProductSpace(PFFT):
             dims[daxes] = 1
         return np.array([0, np.prod(dims)], dtype=int)
 
-    def get_ndiag_slices(self, j=()):
+    def _get_ndiag_slices(self, j=()):
         """Return a sequence of slices for non-diagonal axes and integers for diagonal"""
         assert self.num_components() == 1
         sl = (0,)+self.slice()
@@ -1166,18 +1167,18 @@ class TensorProductSpace(PFFT):
             sa.T[daxes] = j if isinstance(j, int) else np.array(j)[:, None]
         return np.atleast_2d(sa)
 
-    def get_ndiag_slices_and_dims(self, j=()):
+    def _get_ndiag_slices_and_dims(self, j=()):
         """Return nondiagonal slices and their accumulated dimensions"""
-        sl = self.get_ndiag_slices(j)
-        dims = self.get_ndiag_cum_dofs()
+        sl = self._get_ndiag_slices(j)
+        dims = self._get_ndiag_cum_dofs()
         return sl, dims
 
 
 class CompositeSpace:
     """Class for composite tensorproductspaces.
 
-    The mixed spaces are Cartesian products of TensorProductSpaces or
-    other CompositeSpaces.
+    The mixed spaces are Cartesian products of :class:`.TensorProductSpace` or
+    other instances of :class:`.CompositeSpace`.
 
     Parameters
     ----------
@@ -1250,7 +1251,6 @@ class CompositeSpace:
 
     @property
     def dimensions(self):
-        """Return dimension of scalar space"""
         return self.flatten()[0].dimensions
 
     @property
@@ -1299,7 +1299,7 @@ class CompositeSpace:
         return np.concatenate((np.zeros((1, self.dimensions), dtype=int),
                                np.cumsum(self.dims_composite(), axis=0)))
 
-    def get_ndiag_cum_dofs(self):
+    def _get_ndiag_cum_dofs(self):
         """Return the cumulative sum of degrees of freedom along nondiagonal axes"""
         dc = self.dims_composite()
         daxes = self.get_diagonal_axes()
@@ -1308,8 +1308,7 @@ class CompositeSpace:
         dims = np.array([0]+np.cumsum(dci).tolist())
         return dims
 
-    def get_ndiag_slices(self, j=()):
-        nvars = self.num_components()
+    def _get_ndiag_slices(self, j=()):
         sl = self.slice()
         daxes = self.get_diagonal_axes()
         if len(daxes) > 0:
@@ -1321,8 +1320,8 @@ class CompositeSpace:
             sa.T[daxes] = np.array(j)[:, None]
         return sa
 
-    def get_ndiag_slices_and_dims(self, j=()):
-        sl = self.get_ndiag_slices(j)
+    def _get_ndiag_slices_and_dims(self, j=()):
+        sl = self._get_ndiag_slices(j)
         dims = [0]
         for s in sl:
             d = np.prod([1 if isinstance(i, int) else i.stop-i.start for i in s])
@@ -1342,7 +1341,7 @@ class CompositeSpace:
             return np.prod(N)
 
     def shape(self, forward_output=False):
-        """Return shape of arrays for CompositeSpace
+        """Return shape of arrays for :class:`.CompositeSpace`
 
         Parameters
         ----------
@@ -1367,7 +1366,7 @@ class CompositeSpace:
         return s
 
     def global_shape(self, forward_output=False):
-        """Return global shape for CompositeSpace
+        """Return global shape for :class:`.CompositeSpace`
 
         Parameters
         ----------
@@ -1394,7 +1393,6 @@ class CompositeSpace:
         return (slice(None),) + s
 
     def num_components(self):
-        """Return number of spaces in mixed space"""
         f = self.flatten()
         return len(f)
 
@@ -1475,7 +1473,7 @@ class VectorSpace(CompositeSpace):
         return 1
 
     def shape(self, forward_output=False):
-        """Return shape of arrays for VectorSpace
+        """Return shape of arrays for :class:`.VectorSpace`
 
         Parameters
         ----------
@@ -1526,7 +1524,6 @@ class TensorSpace(VectorSpace):
         CompositeSpace.__init__(self, spaces)
 
     def num_components(self):
-        """Return number of spaces in mixed space"""
         return self.dimensions**2
 
     @property
@@ -1634,46 +1631,32 @@ class BoundaryValues:
 
     Parameters
     ----------
-    T : TensorProductSpace
-    bc : tuple of numbers
-        Tuple with physical boundary values at edges of 1D domain
+    T : :class:`.TensorProductSpace`
+    bc : :class:`.BoundaryConditions`
     """
     # pylint: disable=protected-access, redefined-outer-name, dangerous-default-value, unsubscriptable-object
 
-    def __init__(self, T, bc=(0, 0)):
+    def __init__(self, T, bc):
         self.base = T
         self.tensorproductspace = None
-        self.bc = bc            # Containing Data, sp.Exprs or np.ndarray
-        self.bcs = list((0,)*len(bc))       # Processed bc
-        self.bcs_final = list((0,)*len(bc)) # Data. May differ from bcs only for TensorProductSpaces
+        self.bc = bc
+        self.bcs = bc.orderedvals()       # Processed bc
+        self.bcs_final = bc.orderedvals() # Data. May differ from bcs only for TensorProductSpaces
         self.axis = 0
         self.bc_time = 0
-        self.update_bcs(bc=bc)
-
-    def update_bcs(self, bc=None):
-        if bc is not None:
-            assert isinstance(bc, (list, tuple))
-            assert len(bc) in (2, 3, 4, 6, 8)
-            self.bc = list(bc)
-            for i, bci in enumerate(bc):
-                if isinstance(bci, (Number, sp.Expr, np.ndarray)):
-                    self.bcs[i] = bci
-                elif bci is None:
-                    self.bcs[i] = None
-                else:
-                    raise NotImplementedError
-
-            self.bcs_final[:] = self.bcs
 
     def update_bcs_time(self, time):
         tt = sp.symbols('t', real=True)
         update_time = False
-        for i, bci in enumerate(self.bc):
+        self.bcs = self.bc.orderedvals()
+        self.bcs_final = self.bc.orderedvals()
+        for i, bci in enumerate(self.bcs):
             if isinstance(bci, sp.Expr):
                 if tt in bci.free_symbols:
                     self.bc_time = time
                     self.bcs[i] = bci.subs(tt, time)
                     update_time = True
+
         if update_time:
             self.bcs_final[:] = self.bcs
             if self.tensorproductspace is not None:
@@ -1685,7 +1668,6 @@ class BoundaryValues:
         To modify boundary conditions on the fly, modify first self.bc and then
         call this function. The boundary condition can then be applied as before.
         """
-        #from shenfun.utilities import split
         from shenfun import project
 
         self.axis = this_base.axis
@@ -1711,10 +1693,8 @@ class BoundaryValues:
                     if axis is None:
                         number_of_bases_after_this += 1
 
-            self.number_of_bases_after_this = number_of_bases_after_this
-
             if self.has_nonhomogeneous_bcs() is False:
-                for i in range(len(self.bc)):
+                for i in range(len(self.bcs)):
                     self.bcs[i] = self.bcs_final[i] = 0
                 return
 
@@ -1724,9 +1704,9 @@ class BoundaryValues:
 
             b = Array(T)
             s = T.local_slice(False)[self.axis]
-            num_bcs = len(self.bc) - np.count_nonzero(np.array(self.bc) is None)
+            num_bcs = self.bc.num_bcs()
 
-            for j, bci in enumerate(self.bc):
+            for j, bci in enumerate(self.bc.orderedvals()):
                 if isinstance(bci, sp.Expr):
                     X = T.local_mesh(True)
                     for sym in bci.free_symbols:
@@ -1743,26 +1723,17 @@ class BoundaryValues:
 
                     # Put the value in the position of the bc dofs
                     if s.stop == int(this_base.N*this_base.padding_factor):
-                        if num_bcs == 1: # e.g., UpperDirichlet
-                            b[this_base.si[-1]] = f_bci
-                        else:
-                            b[this_base.si[-(len(self.bc))+j]] = f_bci
+                        b[this_base.si[-num_bcs+j]] = f_bci
 
                 elif isinstance(bci, (Number, np.ndarray)):
                     if s.stop == int(this_base.N*this_base.padding_factor):
-                        if num_bcs == 1:
-                            b[this_base.si[-1]] = bci
-                        else:
-                            b[this_base.si[-(len(self.bc))+j]] = bci
-
-                elif bci is None:
-                    continue
+                        b[this_base.si[-num_bcs+j]] = bci
 
                 else:
                     raise NotImplementedError
 
             if len(T.get_nonhomogeneous_axes()) == 1:
-                for j, bci in enumerate(self.bc):
+                for j, bci in enumerate(self.bc.orderedvals()):
                     from .spectralbase import FuncWrap
 
                     if number_of_bases_after_this == 0:
@@ -1780,79 +1751,11 @@ class BoundaryValues:
                     # Now b_hat contains the correct slices in slm1 and slm2
                     # These are the values to use on intermediate steps.
                     # If for example a Dirichlet space is squeezed between two Fourier spaces
-                    if num_bcs == 1:
-                        self.bcs[j] = b_hat[this_base.si[-1]].copy()
-                    else:
-                        self.bcs[j] = b_hat[this_base.si[-(len(self.bc))+j]].copy()
+                    self.bcs[j] = b_hat[this_base.si[-num_bcs+j]].copy()
 
                 b_hat = T.forward(b).copy()
-                for j, bci in enumerate(self.bc):
-                    if num_bcs == 1:
-                        self.bcs_final[j] = b_hat[this_base.si[-1]].copy()
-                    else:
-                        self.bcs_final[j] = b_hat[this_base.si[-(len(self.bc))+j]].copy()
-
-            #msdict = split(T.coors.sg)
-            #assert len(msdict) == 1
-            #_c = msdict[0]['coeff']
-            #if len(T.get_nonhomogeneous_axes()) == 1:
-            #    if number_of_bases_after_this == 0:
-            #        # Dirichlet base is the first to be transformed
-            #        b_hat = b
-
-            #    elif number_of_bases_after_this == 1:
-            #        T.forward._xfftn[0].input_array[...] = b*_c
-
-            #        T.forward._xfftn[0]()
-            #        arrayA = T.forward._xfftn[0].output_array
-            #        arrayB = T.forward._xfftn[1].input_array
-            #        T.forward._transfer[0](arrayA, arrayB)
-            #        b_hat = arrayB.copy()
-
-            #    elif number_of_bases_after_this == 2:
-
-            #        T.forward._xfftn[0].input_array[...] = b*_c
-            #        T.forward._xfftn[0]()
-            #        arrayA = T.forward._xfftn[0].output_array
-            #        arrayB = T.forward._xfftn[1].input_array
-            #        T.forward._transfer[0](arrayA, arrayB)
-
-            #        T.forward._xfftn[1]()
-            #        arrayA = T.forward._xfftn[1].output_array
-            #        arrayB = T.forward._xfftn[2].input_array
-            #        T.forward._transfer[1](arrayA, arrayB)
-            #        b_hat = arrayB.copy()
-
-            #    # Now b_hat contains the correct slices in slm1 and slm2
-            #    # These are the values to use on intermediate steps. If for example the Dirichlet space is squeezed between two Fourier spaces
-            #    for i in range(len(self.bc)):
-            #        if self.bcs[i] is None:
-            #            continue
-            #        if num_bcs == 1:
-            #            self.bcs[i] = b_hat[this_base.si[-1]].copy()
-            #        else:
-            #            self.bcs[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
-
-            #    # Final (the values to set on fully transformed functions)
-
-            #    T.forward._xfftn[0].input_array[...] = b*_c
-
-            #    for i in range(len(T.forward._transfer)):
-            #        T.forward._xfftn[i]()
-            #        arrayA = T.forward._xfftn[i].output_array
-            #        arrayB = T.forward._xfftn[i+1].input_array
-            #        T.forward._transfer[i](arrayA, arrayB)
-
-            #    T.forward._xfftn[-1]()
-            #    b_hat = T.forward._xfftn[-1].output_array
-            #    for i in range(len(self.bc)):
-            #        if self.bcs[i] is None:
-            #            continue
-            #        elif num_bcs == 1:
-            #            self.bcs_final[i] = b_hat[this_base.si[-1]].copy()
-            #        else:
-            #            self.bcs_final[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
-
+                for j in range(self.bc.num_bcs()):
+                    self.bcs_final[j] = b_hat[this_base.si[-num_bcs+j]].copy()
 
             elif len(T.get_nonhomogeneous_axes()) == 2:
                 assert len(T.bases) == 2, 'Only implemented for 2D'
@@ -1863,41 +1766,35 @@ class BoundaryValues:
 
                 other_base = bases[0]
                 ua = Array(other_base)
-                bc_this = this_base.bc.bc.copy()
-                bc_other = other_base.bc.bc.copy()
+                lr = {'left': 0, 'right': 1}
+                num_bcs = this_base.bcs.num_bcs()
                 df = 2./(other_base.domain[1]-other_base.domain[0])
-                for i in range(2): # e.g., x = -1 and then x = 1
-                    bcj = bc_this[i]
-                    for j in range(2): # e.g., y = -1 and then y = 1
-                        xj = other_base.domain[j]
-                        sym = sp.sympify(bcj).free_symbols
-                        if len(sym) == 1:
-                            xx = sym.pop()
-                            if j == 0 and other_base.boundary_condition() == 'NeumannDirichlet':
-                                s = bcj.diff(xx, 1).subs(xx, xj)/df
-                            elif j == 1 and other_base.boundary_condition() == 'DirichletNeumann':
-                                s = bcj.diff(xx, 1).subs(xx, xj)/df
-                            elif j == 0 and other_base.boundary_condition() == 'UpperDirichletNeumann':
-                                xj = other_base.domain[1]
-                                s = bcj.subs(xx, xj)
-                            elif j == 1 and other_base.boundary_condition() == 'UpperDirichletNeumann':
-                                s = bcj.diff(xx, 1).subs(xx, xj)/df
-                            elif other_base.boundary_condition() == 'Neumann':
-                                s = bcj.diff(xx, 1).subs(xx, xj)/df
+                for i, bcj in enumerate(this_base.bcs.orderedvals()):
+                    sym = sp.sympify(bcj).free_symbols
+                    xx = sym.pop() if len(sym) == 1 else None
+                    for lr_other, bco in other_base.bcs.items():
+                        xj = other_base.domain[lr[lr_other]]
+                        for key in bco.keys():
+                            if xx is not None:
+                                if key == 'D':
+                                    bco[key] = bcj.subs(xx, xj)
+                                elif key == 'N':
+                                    bco[key] = bcj.diff(xx, 1).subs(xx, xj)/df
+                                elif key == 'N2':
+                                    bco[key] = bcj.diff(xx, 2).subs(xx, xj)/df**2
+                                elif key == 'N3':
+                                    bco[key] = bcj.diff(xx, 3).subs(xx, xj)/df**3
+                                elif key == 'N4':
+                                    bco[key] = bcj.diff(xx, 4).subs(xx, xj)/df**4
                             else:
-                                s = bcj.subs(xx, xj)
-                            other_base.bc.bc[j] = s
-                            other_base.bc.bcs[j] = s
-                            other_base.bc.bcs_final[j] = s
-                        else:
-                            other_base.bc.bc[j] = bcj
-                            other_base.bc.bcs[j] = bcj
-                            other_base.bc.bcs_final[j] = bcj
-                    ua[:] = b[this_base.si[-(len(self.bc))+i]]
+                                bco[key] = bcj
+                        other_base.bc.bcs[:] = other_base.bcs.orderedvals()
+                        other_base.bc.bcs_final[:] = other_base.bcs.orderedvals()
+                    ua[:] = b[this_base.si[-num_bcs+i]]
                     self.bcs_final[i] = project(ua, other_base)
 
-    def add_to_orthogonal(self, u, uh):
-        """Add contribution from boundary functions to `u`
+    def _add_to_orthogonal(self, u, uh):
+        """Add contribution from boundary functions to ``u``
 
         Parameters
         ----------
@@ -1917,7 +1814,7 @@ class BoundaryValues:
         for i, row in enumerate(M):
             u[self.base.si[i]] += np.sum(self.base.broadcast_to_ndims(row)*uh[sl], axis=self.base.axis)
 
-    def add_mass_rhs(self, u):
+    def _add_mass_rhs(self, u):
         """Add contribution of boundary functions to rhs before solving in forward transforms
 
         Parameters
@@ -1948,29 +1845,16 @@ class BoundaryValues:
             must be a fully transformed Function.
 
         """
-
-        M = len(self.bc)
-        num_bcs = len(self.bc) - np.count_nonzero(np.array(self.bc) == None)
-
+        M = self.bc.num_bcs()
         if final is True:
             for i in range(M):
-                if num_bcs == 1:
-                    if self.bcs_final[i] is None:
-                        continue
-                    u[self.base.si[-1]] = self.bcs_final[i]
-                else:
-                    u[self.base.si[-(M)+i]] = self.bcs_final[i]
-
+                u[self.base.si[-(M)+i]] = self.bcs_final[i]
         else:
             for i in range(M):
-                if num_bcs == 1:
-                    if self.bcs[i] is None:
-                        continue
-                    u[self.base.si[-1]] = self.bcs[i]
                 u[self.base.si[-(M)+i]] = self.bcs[i]
 
     def has_nonhomogeneous_bcs(self):
-        for bc in self.bc:
+        for bc in self.bc.orderedvals():
             if isinstance(bc, Number):
                 if not bc == 0:
                     return True
