@@ -386,7 +386,34 @@ def integrate_sympy(f, d):
         #return sp.Integral(f, d).evalf()
         return sp.integrate(f, d).evalf()
 
-def split(measures):
+def rsplit(measures):
+    d = []
+    def _split(ms):
+        ms = sp.sympify(ms)
+        if isinstance(ms, Number):
+            d = {'coeff': ms}
+        elif len(ms.free_symbols) == 0:
+            d = {'coeff': ms.evalf()}
+        else:
+            d = sp.separatevars(ms.factor(), dict=True)
+        if d is None:
+            return d
+        d = defaultdict(lambda: 1, {str(k): v for k, v in d.items()})
+        dc = d['coeff']
+        d['coeff'] = int(dc) if isinstance(dc, (sp.Integer, int)) else float(dc)
+        return d
+
+    ms = sp.sympify(measures)
+    di = _split(ms)
+    if di is not None:
+        d += [di]
+    else:
+        ms = ms.expand()
+        for arg in ms.args:
+            d += split(arg)
+    return d
+
+def split(measures, expand=False):
 
     def _split(ms):
         ms = sp.sympify(ms)
@@ -399,14 +426,25 @@ def split(measures):
         if d is None:
             raise RuntimeError('Could not split', ms)
         d = defaultdict(lambda: 1, {str(k): v for k, v in d.items()})
-        dc = d['coeff']
-        d['coeff'] = int(dc) if isinstance(dc, (sp.Integer, int)) else float(dc)
+        dc = sp.sympify(d['coeff'])
+        if dc.is_integer:
+            d['coeff'] = int(dc)
+        elif dc.is_real:
+            d['coeff'] = float(dc)
+        elif dc.is_complex:
+            d['coeff'] = complex(dc)
+        else:
+            raise RuntimeError
         return d
 
     ms = sp.sympify(measures)
-    if not config['assembly']['splitmeasure']:
-        if len(ms.free_symbols) <= 1:
-            return [_split(ms)]
+    if not expand:
+        try:
+            di = _split(ms)
+            if di is not None:
+                return [di]
+        except RuntimeError:
+            pass
 
     ms = ms.expand()
     result = []
